/***************************************************************************
                          configuration.cpp  -  description
                             -------------------
    begin                : Tue Jun 11 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <qpushbutton.h>
#include <qfile.h>
#include <qlistview.h>
#include <qregexp.h>
#include <qlineedit.h>
#include <qtabwidget.h>

#include <klistview.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kconfig.h>
#include <kurlrequester.h>
#include <kfiledialog.h>
#include <kstddirs.h>
#include <ktempfile.h>
#include <knuminput.h>
#include <kdebug.h>

#include "configuration.h"
#include "sanetracker.h"
#include "configdialog.h"
#include "midi.h"
#include "mididriver.h"

Configuration::Configuration(SanetrackerApp* parent)
  : _parent(parent),
    _dialog(0)
{
  _kconfig = kapp->config();
}

Configuration::~Configuration()
{
}

void Configuration::load()
{
  _portInfos = MIDIDriver::portInfos();

  readInstrDef(KURL(locateLocal("appdata", "instruments")), false);

  _kconfig->setGroup("General Options");

  bool bViewToolbar = _kconfig->readBoolEntry("Show Toolbar", true);
  _parent->viewToolBar->setChecked(bViewToolbar);
  _parent->slotViewToolBar();

  bool bViewStatusbar = _kconfig->readBoolEntry("Show Statusbar", true);
  _parent->viewStatusBar->setChecked(bViewStatusbar);
  _parent->slotViewStatusBar();

  KToolBar::BarPosition toolBarPos;
  toolBarPos = (KToolBar::BarPosition) _kconfig->readNumEntry("ToolBarPos", KToolBar::Top);
  _parent->toolBar("mainToolBar")->setBarPos(toolBarPos);

  _parent->fileOpenRecent->loadEntries(_kconfig, "Recent Files");

  QSize size = _kconfig->readSizeEntry("Geometry");

  if(size.isEmpty())
    size.setWidth(640), size.setHeight(480);

  _parent->resize(size);

  QStringList ports = _kconfig->groupList().grep(QRegExp("^Port [0-9]*"));

  _ports.clear();
  _ports.resize(ports.size());

  QStringList names;

  for(uint i = 0; i < ports.size(); ++i)
  {
    _kconfig->setGroup(ports[i]);

    PortInfo portInfo;
    portInfo.type = static_cast<PortInfo::Type>(_kconfig->readNumEntry("Type"));
    portInfo.name = _kconfig->readEntry("Name");

    if(portInfo.type == PortInfo::ALSA || portInfo.type == PortInfo::aRts)
    {
      portInfo.client = _kconfig->readNumEntry("Client");
      portInfo.port = _kconfig->readNumEntry("Port");
    }

    Port* port = new Port(portInfo);

    for(int id = 0; id < MIDI_CHANNELS_MAX; ++id)
    {
      QString name = _kconfig->readEntry(QString("Dev%1Name").arg(id));

      if(name.isNull())
        continue;

      if(!names.grep(name).empty())
      {
        uint i = 2;

        while(!names.grep(name + QString(" (%1)").arg(i)).empty())
          ++i;

        name += QString(" (%1)").arg(i);
      }

      names.push_back(name);

      port->device(id).setName(name);
      port->device(id).setType(_kconfig->readEntry(QString("Dev%1Type").arg(id)));
    }

    _ports.insert(i, port);
  }
}

void Configuration::save()
{
  _kconfig->setGroup("General Options");
  _kconfig->writeEntry("Geometry", _parent->size());
  _kconfig->writeEntry("Show Toolbar", _parent->viewToolBar->isChecked());
  _kconfig->writeEntry("Show Statusbar", _parent->viewStatusBar->isChecked());
  _kconfig->writeEntry("ToolBarPos", (int) _parent->toolBar("mainToolBar")->barPos());

  _parent->fileOpenRecent->saveEntries(_kconfig, "Recent Files");

  writeInstrDef(KURL(locateLocal("appdata", "instruments")));

  QStringList ports = _kconfig->groupList().grep(QRegExp("^Port [0-9]*"));

  for(uint i = 0; i < ports.size(); ++i)
    _kconfig->deleteGroup(ports[i]);

  for(uint i = 0; i < _ports.size(); ++i)
  {
    _kconfig->setGroup(QString("Port %1").arg(i));

    _kconfig->writeEntry("Type", (int) _ports[i]->type());
    _kconfig->writeEntry("Name", _ports[i]->deviceFile());

    if(_ports[i]->type() == PortInfo::ALSA || _ports[i]->type() == PortInfo::aRts)
    {
      _kconfig->writeEntry("Client", _ports[i]->client());
      _kconfig->writeEntry("Port", _ports[i]->port());
    }

    for(int id = 0; id < MIDI_CHANNELS_MAX; ++id)
    {
      if(_ports[i]->device(id).name().isEmpty())
        continue;

      _kconfig->writeEntry(QString("Dev%1Name").arg(id), _ports[i]->device(id).name());
      _kconfig->writeEntry(QString("Dev%1Type").arg(id), _ports[i]->device(id).type());
    }
  }
}

void Configuration::configure(int defaultTab, bool dirty)
{
  if(_dialog)
    return;

  _dialog = new ConfigDialog(_parent, 0, FALSE, Qt::WDestructiveClose);

  connect(_dialog->buttonOk, SIGNAL(clicked()), SLOT(slotOkClicked()));
  connect(_dialog->buttonCancel, SIGNAL(clicked()), SLOT(slotCancelClicked()));
  connect(_dialog->buttonApply, SIGNAL(clicked()), SLOT(slotApplyClicked()));
  connect(_dialog->portType, SIGNAL(activated(int)), SLOT(slotPortTypeActivated(int)));
  connect(_dialog->listPorts, SIGNAL(selectionChanged()), SLOT(slotPortListSelChanged()));
  connect(_dialog->buttonAddPort, SIGNAL(clicked()), SLOT(slotAddPortClicked()));
  connect(_dialog->buttonRemovePort, SIGNAL(clicked()), SLOT(slotRemovePortClicked()));
  connect(_dialog->listDevices, SIGNAL(selectionChanged()), SLOT(slotDeviceListSelChanged()));
  connect(_dialog->buttonAddDevice, SIGNAL(clicked()), SLOT(slotAddDeviceClicked()));
  connect(_dialog->buttonRemoveDevice, SIGNAL(clicked()), SLOT(slotRemoveDeviceClicked()));
  connect(_dialog->buttonImportType, SIGNAL(clicked()), SLOT(slotImportTypeClicked()));
  connect(_dialog->typeURL, SIGNAL(textChanged(const QString&)), SLOT(slotTypeURLChanged(const QString&)));
  connect(_dialog, SIGNAL(destroyed()), SLOT(slotDestroyed()));
  _dialog->typeURL->fileDialog()->setMode(KFile::File | KFile::ExistingOnly);

  _dialog->listPorts->setColumnText(0, i18n("Port Type"));
  _dialog->listPorts->addColumn(i18n("Device"));
  _dialog->listPorts->clear();
  _listPortsValues.clear();

  _dialog->listDevices->setColumnText(0, i18n("Port"));
  _dialog->listDevices->addColumn(i18n("ID"));
  _dialog->listDevices->addColumn(i18n("Type"));
  _dialog->listDevices->addColumn(i18n("Name"));
  _dialog->listDevices->clear();

  for(uint i = 0; i < _ports.size(); ++i)
  {
    const Port& port = *_ports[i];

    (void) new QListViewItem(_dialog->listPorts, 
        (port.type() == PortInfo::Raw)          ? i18n("Raw")
      : (port.type() == PortInfo::ALSA)         ? i18n("ALSA")
      : (port.type() == PortInfo::aRts)         ? i18n("aRts")
                                                : i18n("Unknown"), 
      port.deviceFile());
  
    _listPortsValues.append(port.info());

    if(port.output())
      _dialog->attachedPort->insertItem(port.deviceFile());

    for(int id = 0; id < MIDI_CHANNELS_MAX; ++id)
    {
      if(port.device(id).name().isNull())
        continue;

      (void) new QListViewItem(_dialog->listDevices,
        port.deviceFile(),
        QString("%1").arg(id + 1),
        port.device(id).type(),
        port.device(id).name());
    }
  }

  _dialog->listTypes->setColumnText(0, i18n("Instrument type"));
  _dialog->listTypes->clear();

  QMapConstIterator<QString, Instrument> instr;

  for(instr = _instruments.begin(); instr != _instruments.end(); ++instr)
  {
    (void) new QListViewItem(_dialog->listTypes, instr.key());

    _dialog->deviceType->insertItem(instr.key());
  }

  _dialog->listTypes->sort();

  if(!dirty)
    _dialog->buttonApply->setEnabled(false);

  _dialog->tabWidget->setCurrentPage(defaultTab);

  _dialog->show();
}

void Configuration::readInstrDef(const KURL& url, bool showErrors)
{
  QFile file;

  if(url.isLocalFile())
  {
    file.setName(url.directory(false) + url.fileName());

    if(!file.open(IO_ReadOnly))
    {
      // XXX: Give a better error message.
      if(showErrors)
        KMessageBox::error(_parent, i18n("Unable to open instrument definition file"));

      return;
    }
  }
  else
  {
    QString tmpfile;

#if KDE_VERSION < KDE_MAKE_VERSION(3,2,0)
    if(KIO::NetAccess::download(url, tmpfile))
#else
    if(KIO::NetAccess::download(url, tmpfile, 0))
#endif
    {
      if(showErrors)
        KMessageBox::error(_parent, KIO::NetAccess::lastErrorString());

      return;
    }

    file.setName(tmpfile);
    file.open(IO_ReadOnly);
    KIO::NetAccess::removeTempFile(tmpfile);
  }

  QTextStream stream(&file);

  QString line;

  enum Section
  {
    ControllerNames, InstrumentDefinitions, NoteNames, NRPNNames, PatchNames, RPNNames, Unknown
  };

  Section section = Unknown;
  QString subSection = "";

  int lineno = 0;

  while(line = stream.readLine())
  {
    lineno++;

    line.stripWhiteSpace();

    // Skip comments and empty lines
    if(line.startsWith(";") || line.isEmpty())
      continue;

    if(line.startsWith("."))
    {
      section = (line == ".Controller Names")       ? ControllerNames
              : (line == ".Instrument Definitions") ? InstrumentDefinitions
              : (line == ".Note Names")             ? NoteNames
              : (line == ".NRPN Names")             ? NRPNNames
              : (line == ".Patch Names")            ? PatchNames
              : (line == ".RPN Names")              ? RPNNames
                                                    : Unknown;
    }
    else if(line.startsWith("["))
    {
      if(!line.endsWith("]"))
      {
        KMessageBox::error(_parent,
          i18n("Unmateched [ on line %1 in instrument "
               "defintinion file.").arg(lineno));

        break;
      }

      subSection = line.mid(1, line.length() - 2);
    }
    else
    {
      int pos = line.find('=');

      if(pos == -1)
      {
        KMessageBox::error(_parent,
          i18n("Syntax error on line %1 in instrument "
               "definition file.").arg(lineno));

        break;
      }

      QString value = line.right(line.length() - pos - 1);
      line.truncate(pos);
      QString& key = line;

      key.stripWhiteSpace();
      value.stripWhiteSpace();

      QString index, subIndex;

      pos = key.find('[');

      if(pos != -1)
      {
        index = key.mid(pos + 1, key.length() - pos - 2);
        key.truncate(pos);

        key.stripWhiteSpace();
        index.stripWhiteSpace();

        pos = index.find(',');

        if(pos != -1)
        {
          subIndex = index.mid(pos + 1, index.length() - pos - 1);
          index.truncate(pos);

          index.stripWhiteSpace();
          subIndex.stripWhiteSpace();
        }
      }

      switch(section)
      {
      case ControllerNames:

        _controls[subSection][key.toInt()] = value;

        break;

      case InstrumentDefinitions:

        if(key == "BankSelMethod")
        {
          _instruments[subSection].setBankSelMethod((Instrument::BankSelMethod) value.toInt());
        }
        else if(key == "Patch")
        {
          _instruments[subSection].setBank((index == "*") ? -1 : index.toInt(), value);
        }
        else if(key == "Control")
        {
          _instruments[subSection].setControls(value);
        }
        else if(key == "NRPN")
        {
          _instruments[subSection].setNRPNs(value);
        }
        else if(key == "RPN")
        {
          _instruments[subSection].setRPNs(value);
        }
        else if(key == "Drum")
        {
          _instruments[subSection].setDrums(
            (index == "*") ? -1 : index.toInt(),
            (subIndex == "*") ? -1 : subIndex.toInt(),
            (value.toInt() == 1));
        }
        else if(key == "Key")
        {
          _instruments[subSection].setKeys(
            (index == "*") ? -1 : index.toInt(),
            (subIndex == "*") ? -1 : subIndex.toInt(),
            value);
        }

        break;

      case NoteNames:

        _notes[subSection][key.toInt()] = value;

        break;

      case NRPNNames:

        _nrpns[subSection][key.toInt()] = value;

        break;

      case PatchNames:

        _banks[subSection][key.toInt()] = value;

        break;

      case RPNNames:

        _rpns[subSection][key.toInt()] = value;

        break;

      default:
      case Unknown:

        ;
      }
    }
  }

  file.close();

  QMap<QString, Instrument>::Iterator instr;

  for(instr = _instruments.begin(); instr != _instruments.end(); ++instr)
  {
    instr.data().buildPatchList(_banks);
  }
}

void Configuration::Instrument::buildPatchList(const BankMap& banks)
{
  if(!patchList().empty())
    return;

  QMap<int, QString>::ConstIterator i;

  for(i = _banks.begin(); i != _banks.end(); ++i)
  {
    const Bank& bank = banks[i.data()];

    Bank::ConstIterator j;

    for(j = bank.begin(); j != bank.end(); ++j)
    {
      _patchList.push_back(i.key() << 7 | j.key());
    }
  }
}

void Configuration::writeInstrDef(const KURL& url)
{
  KTempFile* tmpfile = new KTempFile;

  QTextStream* stream = new QTextStream(tmpfile->file());

  *stream << ".Patch Names" << endl;
  *stream << endl;

  QMapConstIterator<QString, Bank> bank;

  for(bank = _banks.begin(); bank != _banks.end(); ++bank)
  {
    *stream << "[" << bank.key() << "]" << endl;

    Bank::const_iterator patch;

    for(patch = bank.data().begin(); patch != bank.data().end(); ++patch)
    {
      *stream << patch.key() << "=" << patch.data() << endl;
    }

    *stream << endl;
  }

  *stream << ".Note Names" << endl;
  *stream << endl;

  QMapConstIterator<QString, Notes> notes;

  for(notes = _notes.begin(); notes != _notes.end(); ++notes)
  {
    *stream << "[" << notes.key() << "]" << endl;

    Notes::const_iterator note;

    for(note = notes.data().begin(); note != notes.data().end(); ++note)
    {
      *stream << note.key() << "=" << note.data() << endl;
    }

    *stream << endl;
  }

  *stream << ".Controller Names" << endl;
  *stream << endl;

  QMapConstIterator<QString, Control> controls;

  for(controls = _controls.begin(); controls != _controls.end(); ++controls)
  {
    *stream << "[" << controls.key() << "]" << endl;

    Control::const_iterator control;

    for(control = controls.data().begin(); control != controls.data().end(); ++control)
    {
      *stream << control.key() << "=" << control.data() << endl;
    }

    *stream << endl;
  }

  *stream << ".NRPN Names" << endl;
  *stream << endl;

  QMapConstIterator<QString, NRPNs> nrpns;

  for(nrpns = _nrpns.begin(); nrpns != _nrpns.end(); ++nrpns)
  {
    *stream << "[" << nrpns.key() << "]" << endl;

    NRPNs::const_iterator nrpn;

    for(nrpn = nrpns.data().begin(); nrpn != nrpns.data().end(); ++nrpn)
    {
      *stream << nrpn.key() << "=" << nrpn.data() << endl;
    }

    *stream << endl;
  }

  *stream << ".RPN Names" << endl;
  *stream << endl;

  QMapConstIterator<QString, RPNs> rpns;

  for(rpns = _rpns.begin(); rpns != _rpns.end(); ++rpns)
  {
    *stream << "[" << rpns.key() << "]" << endl;

    RPNs::const_iterator rpn;

    for(rpn = rpns.data().begin(); rpn != rpns.data().end(); ++rpn)
    {
      *stream << rpn.key() << "=" << rpn.data() << endl;
    }

    *stream << endl;
  }

  *stream << ".Instrument Definitions" << endl;
  *stream << endl;

  QMapConstIterator<QString, Instrument> instr;

  for(instr = _instruments.begin(); instr != _instruments.end(); ++instr)
  {
    *stream << "[" << instr.key() << "]" << endl;
    *stream << "BankSelMethod=" << (int) instr.data().bankSelMethod() << endl;

    QMapConstIterator<int, QString> bank;

    for(bank = instr.data().banks().begin(); bank != instr.data().banks().end(); ++bank)
    {
      *stream << "Patch[";
      if(bank.key() == -1)
        *stream << "*";
      else
        *stream << bank.key();
      *stream << "]=" << bank.data() << endl;
    }

    QMapConstIterator<int, QMap<int, bool> > isDrums;

    for(isDrums = instr.data().isDrums().begin(); isDrums != instr.data().isDrums().end(); ++isDrums)
    {
      QMapConstIterator<int, bool> isDrums2;

      for(isDrums2 = isDrums.data().begin(); isDrums2 != isDrums.data().end(); ++isDrums2)
      {
        *stream << "Drum[";
        if(isDrums2.key() == -1)
          *stream << "*";
        else
          *stream << isDrums2.key();
        *stream << ",";
        if(isDrums2.key() == -1)
          *stream << "*";
        else
          *stream << isDrums2.key();
        *stream << "]=" << (isDrums2.data() ? "1" : "0") << endl;
      }
    }

    QMapConstIterator<int, QMap<int, QString> > keys;

    for(keys = instr.data().keys().begin(); keys != instr.data().keys().end(); ++keys)
    {
      QMapConstIterator<int, QString> key;

      for(key = keys.data().begin(); key != keys.data().end(); ++key)
      {
        *stream << "Key[";
        if(keys.key() == -1)
          *stream << "*";
        else
          *stream << keys.key();
        *stream << ",";
        if(key.key() == -1)
          *stream << "*";
        else
          *stream << key.key();
        *stream << "]=" << key.data() << endl;
      }
    }

    if(!instr.data().controls().isEmpty())
    {
      *stream << "Control=" << instr.data().controls() << endl;
    }

    if(!instr.data().nrpns().isEmpty())
    {
      *stream << "NRPN=" << instr.data().nrpns() << endl;
    }

    if(!instr.data().rpns().isEmpty())
    {
      *stream << "RPN=" << instr.data().rpns() << endl;
    }

    *stream << endl;
  }

  delete stream;

  tmpfile->close();

#if KDE_VERSION < KDE_MAKE_VERSION(3,2,0)
  KIO::NetAccess::upload(tmpfile->name(), url);
#else
  KIO::NetAccess::upload(tmpfile->name(), url, 0);
#endif

  tmpfile->unlink();

  delete tmpfile;
}

void Configuration::slotOkClicked()
{
  if(_dialog->buttonApply->isEnabled())
    slotApplyClicked();
  slotCancelClicked();
}

void Configuration::slotApplyClicked()
{
  _ports.clear();
  _ports.resize(_listPortsValues.size());

  PortInfos::const_iterator portInfo;
  int number = 0;

  for(portInfo = _listPortsValues.begin(); portInfo != _listPortsValues.end(); ++portInfo, ++number)
  {
    QListViewItem* device;

    Port* port = new Port(*portInfo);

    for(device = _dialog->listDevices->firstChild(); device; device = device->nextSibling())
    {
      if(device->text(0) != (*portInfo).name)
        continue;

      int id = device->text(1).toInt() - 1;

      port->device(id).setType(device->text(2));
      port->device(id).setName(device->text(3));
    }

    _ports.insert(number, port);
  }

  _parent->reconfigure();

  save();

  _dialog->buttonApply->setEnabled(false);
}

void Configuration::slotCancelClicked()
{
  delete _dialog;
}

void Configuration::slotDestroyed()
{
  _dialog = NULL;
}

void Configuration::slotImportTypeClicked()
{
  readInstrDef(_dialog->typeURL->url());

  _dialog->listTypes->clear();
  _dialog->deviceType->clear();

  QMapConstIterator<QString, Instrument> instr;

  for(instr = _instruments.begin(); instr != _instruments.end(); ++instr)
  {
    (void) new QListViewItem(_dialog->listTypes, instr.key());

    _dialog->deviceType->insertItem(instr.key());
  }

  _dialog->listTypes->sort();
}

void Configuration::slotAddPortClicked()
{
  PortInfo::Type type = static_cast<PortInfo::Type>(_dialog->portType->currentItem());

  PortInfo portInfo;

  if(type == PortInfo::ALSA
  || type == PortInfo::aRts)
  {
    portInfo = _portNameValues[_dialog->portName->currentItem()];
  }
  else
  {
    if(access(_dialog->portFile->text(), R_OK | W_OK))
    {
      KMessageBox::error(_parent,
        i18n("You don't have read and write permissions to the port device "
             "file ('%1').").arg(_dialog->portFile->text()));

      return;
    }

    portInfo.type = type;
    portInfo.name = _dialog->portFile->text();
    portInfo.input = true;
    portInfo.output = true;
  }

  (void) new QListViewItem(_dialog->listPorts, 
      (type == PortInfo::Raw)          ? i18n("Raw")
    : (type == PortInfo::ALSA)         ? i18n("ALSA")
    : /* type == PortInfo::aRts */       i18n("aRts"),
    portInfo.name);

  _listPortsValues.append(portInfo);

  if(portInfo.output)
    _dialog->attachedPort->insertItem(portInfo.name);

  _dialog->buttonApply->setEnabled(true);
}

void Configuration::slotRemovePortClicked()
{
  QListViewItem* item = _dialog->listPorts->currentItem();

  int deviceCount = 0;

  QListViewItem* i;

  for(i = _dialog->listDevices->firstChild(); i; i = i->nextSibling())
  {
    if(i->text(0) == item->text(1))
      deviceCount++;
  }

  if(deviceCount)
  {
    if(KMessageBox::No == KMessageBox::warningYesNo(_parent,
        (deviceCount == 1)
      ? i18n("There is a device attached to this port.  It will be "
             "removed if you remove the port. Proceed?")
      : i18n("There are %1 devices attached to this port. These will be "
             "removed if you removed the port. Proceed?").arg(deviceCount)))
      return;

    for(i = _dialog->listDevices->firstChild(); i; )
    {
      QListViewItem* next = i->nextSibling();

      if(i->text(0) == item->text(1))
        delete i;

      i = next;
    }
  }

  for(PortInfos::iterator port = _listPortsValues.begin(); /* port != _listPortsValues.end() */; ++port)
  {
    if((*port).name == item->text(1))
    {
      _listPortsValues.erase(port);

      break;
    }
  }

  for(int n = 0; n < _dialog->attachedPort->count(); ++n, ++i)
  {
    if(_dialog->attachedPort->text(n) == item->text(1))
    {
      _dialog->attachedPort->removeItem(n);

      break;
    }
  }

  delete item;

  _dialog->buttonApply->setEnabled(true);
}

void Configuration::slotAddDeviceClicked()
{
  const QString& port = _dialog->attachedPort->currentText();
  int id = _dialog->deviceID->value();
  const QString& type = _dialog->deviceType->currentText();
  QString name = _dialog->deviceName->text();

  if(name.isEmpty())
    name = type;

  QStringList names;
  QListViewItem* i;

  for(i = _dialog->listDevices->firstChild(); i; i = i->nextSibling())
  {
    if(i->text(0) == port && i->text(1).toInt() == id)
    {
      if(KMessageBox::No == KMessageBox::warningYesNo(_parent,
        i18n("There already is a device connected to the specified "
             "port having the specified ID. Do you want to replace it?")))
        return;

      delete i;

      break;
    }

    names.push_back(i->text(3));
  }

  if(!names.grep(name).empty())
  {
    uint i = 2;

    while(!names.grep(name + QString(" (%1)").arg(i)).empty())
      ++i;

    name += QString(" (%1)").arg(i);
  }

  (void) new QListViewItem(_dialog->listDevices,
    port,
    QString("%1").arg(id),
    type,
    name);

  _dialog->buttonApply->setEnabled(true);
}

void Configuration::slotRemoveDeviceClicked()
{
  delete _dialog->listDevices->currentItem();

  _dialog->buttonApply->setEnabled(true);
}

void Configuration::slotTypeURLChanged(const QString& url)
{
  if(!url.isEmpty())
    _dialog->buttonImportType->setEnabled(true);
  else
    _dialog->buttonImportType->setEnabled(false);
}

void Configuration::slotPortListSelChanged()
{
  if(_dialog->listPorts->selectedItem())
    _dialog->buttonRemovePort->setEnabled(true);
  else
    _dialog->buttonRemovePort->setEnabled(false);
}

void Configuration::slotDeviceListSelChanged()
{
  if(_dialog->listDevices->selectedItem())
    _dialog->buttonRemoveDevice->setEnabled(true);
  else
    _dialog->buttonRemoveDevice->setEnabled(false);
}

void Configuration::slotPortTypeActivated(int num)
{
  switch(static_cast<PortInfo::Type>(num))
  {
  // Drivers that don't use device files
  case PortInfo::ALSA:
  case PortInfo::aRts:

    _dialog->portFile->setEnabled(false);
    _dialog->portName->setEnabled(true);

    _dialog->portName->clear();
    _portNameValues.clear();

    for(PortInfos::const_iterator port = _portInfos.begin(); port != _portInfos.end(); ++port)
      if((*port).type == static_cast<PortInfo::Type>(num))
      {
        _dialog->portName->insertItem((*port).name);
        _portNameValues.append(*port);
      }
    
    break;

  default:

    _dialog->portFile->setEnabled(true);
    _dialog->portName->setEnabled(false);
  }
}

