/***************************************************************************
                          scheduler.cpp  -  description
                             -------------------
    begin                : Mon Jun 10 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>

#include "scheduler.h"
#include "sanetracker.h"
#include "sanetrackerview.h"
#include "sanetrackerdoc.h"
#include "sequence.h"
#include "mididriver.h"
#include "piano.h"
#include "midi.h"

Scheduler::Scheduler(SanetrackerApp* parent, const Configuration& config)
  : parent(parent),
    playing(false),
    stopped(true),
    _config(config)
{
}

Scheduler::~Scheduler()
{
}

void Scheduler::run()
{
  stopped = false;

  if(!MIDIDriver::portCount())
  {
    kdWarning() << "No MIDI ports found." << endl;

    return;
  }

  const SanetrackerDoc& doc = *parent->getDocument();

  uint status = 0;

  for(;;)
  {
    for(;!stopped;)
    {
      int port;
      int data0 = MIDIDriver::get(port);

      if(data0 >= 0xF8)
      {
        if(data0 == MIDI_CLOCK)
          break;
          
        continue;
      }      
      else if(data0 >= 0x80)
      {
        status = data0;
        
        if(status <= 0xF0)
        {
          data0 = MIDIDriver::get(port);
        }
      }
      
      if(status >= 0x80 && status <= 0xBF || (status & 0xF0) == 0xE0)
      {
        uint data1;
        data1 = MIDIDriver::get(port);
        
        postEvent(parent, new MIDIMessageEvent(status, data0, data1));
      }
      else if(status >= 0xC0 && status <= 0xDF)
      {
        postEvent(parent, new MIDIMessageEvent(status, data0));
      }
      else if(status == 0xF0)
      {
        while(data0 != 0xF7)
        {
          data0 = MIDIDriver::get(port);
        }
      }
    }

    if(stopped)
      break;

    if(playing)
    {
      for(uint i = 0; i < doc.size(); ++i)
      {
        const SanetrackerDoc::Device& device = doc[i];

        if(false /* XXX: is pattern synth? */
        && measureTick == (doc.measureLength(_measure) * 4 / 3))
        {
          setPatch(i, doc[i].sequence(_measure) - 1);
        }
        else if(measureTick == 0)
        {
          setPatch(i, doc[i][doc[i].sequence(_measure)].patch());
        }

        const SanetrackerDoc::Pattern& pattern = doc[i][doc[i].sequence(_measure)];

        for(uint j = 0; j < pattern.size(); ++j)
        {
          const SanetrackerDoc::Pattern::Track& track = pattern[j];

          if(track[measureTick].isEmpty())
            continue;

          if(notes[i].count(j))
          {
            MIDIDriver::put(device.portIndex(), MIDI_NOTE_ON(device.deviceID()));
            MIDIDriver::put(device.portIndex(), notes[i][j]);
            MIDIDriver::put(device.portIndex(), 0);

            notes[i].erase(j);
          }

          if(!track[measureTick].isStop())
          {
            MIDIDriver::put(device.portIndex(), MIDI_NOTE_ON(device.deviceID()));
            MIDIDriver::put(device.portIndex(), track[measureTick].note());
            // XXX: handle relativity
            MIDIDriver::put(device.portIndex(), track[measureTick].velocity());

            notes[i][j] = track[measureTick].note();
          }
        }
      }

      if(!wasPlaying)
      {
        wasPlaying = true;

        MIDIDriver::broadcast(MIDI_PLAY);
      }

      ++tick;

      if(++measureTick == parent->getDocument()->measureLength(_measure))
      {
        ++_measure;
        measureTick = 0;
        postEvent(parent, new PositionEvent);

        if(_measure >= parent->getDocument()->songLength())
          playing = false;
      }
    }
    else if(wasPlaying)
    {
      MIDIDriver::broadcast(MIDI_STOP);

      bool end = (_measure == doc.songLength());

      for(uint i = 0; i < doc.size(); ++i)
      {
        const SanetrackerDoc::Device& device = doc[i];

        MIDIDriver::put(device.portIndex(), MIDI_CONTROL(device.deviceID()));
        MIDIDriver::put(device.portIndex(), end ? MIDI_ALL_NOTES_OFF : MIDI_ALL_SOUND_OFF);
        MIDIDriver::put(device.portIndex(), 0);
      }

      wasPlaying = false;

      postEvent(parent, new PositionEvent);
    }

    MIDIDriver::broadcast(MIDI_CLOCK);
    MIDIDriver::clockStep();
  }
}

void Scheduler::stop()
{
  if(stopped)
    return;

  stopped = true;
  wait();
}

void Scheduler::play(uint measure, uint measureTick)
{
  if(measure >= parent->getDocument()->songLength())
    return;

  this->_measure = measure;
  this->measureTick = measureTick;

  this->tick = 0;

  for(uint i = 0; i < _measure; ++i)
    this->tick += parent->getDocument()->measureLength(i);

  // XXX: set devices' patches

  playing = true;

  postEvent(parent, new PositionEvent);
}

void Scheduler::stopPlayback()
{
  playing = false;

  postEvent(parent, new PositionEvent);
}

void Scheduler::setPatch(uint device, uint patch)
{
  if(patches[device] == patch || patch == 0)
    return;
 
  patches[device] = patch;

  const SanetrackerDoc::Device& dev = (*parent->getDocument())[device];

  const Configuration::Instrument& instr = _config.instrument(dev.configType());

  int physPatch = instr.patchList()[patch - 1];

  MIDIDriver::putRT(dev.portIndex(), MIDI_CONTROL(dev.deviceID()));
  MIDIDriver::putRT(dev.portIndex(), MIDI_BANK_SELECT_FINE);
  MIDIDriver::putRT(dev.portIndex(), (physPatch >> 7) & 0x7F);

  MIDIDriver::putRT(dev.portIndex(), MIDI_CONTROL(dev.deviceID()));
  MIDIDriver::putRT(dev.portIndex(), MIDI_BANK_SELECT);
  MIDIDriver::putRT(dev.portIndex(), physPatch >> 14);
  
  MIDIDriver::putRT(dev.portIndex(), MIDI_PATCH_CHANGE(dev.deviceID()));
  MIDIDriver::putRT(dev.portIndex(), physPatch & 0x7F);
}

