#include "artstracker.h"
#include "sequenceutils.h"

using namespace std;

namespace Arts {
namespace Tracker {

class Track_impl : virtual public Track_skel {
protected:
	vector<Note> _notes;
public:
	Track_impl()
	{	
		for(int i=0;i<64;i++)
			_notes.push_back(Note(0,0,0,0,0));
	}
	vector<Note> *notes()
	{
		return new vector<Note>(_notes);
	}
	Note getNote(long pos)
	{
		assert(pos >= 0);
		assert(pos < (long)_notes.size());
		return _notes[pos];
	}
	void setNote(long pos, const Note& note)
	{
		assert(pos >= 0);
		assert(pos < (long)_notes.size());
		_notes[pos] = note;
	}
	Note loadNote(const vector<string>& list)
	{
		Note result(0,0,0,0,0);

		unsigned long i;
		string cmd, param;

		for(i=0;i<list.size();i++)
		{
			if(parse_line(list[i],cmd,param))	// otherwise: empty or comment
			{
				if(cmd == "note")
					result.note = atoi(param.c_str());
				if(cmd == "instrument")
					result.instrument = atoi(param.c_str());
				if(cmd == "velocity")
					result.velocity = atoi(param.c_str());
				if(cmd == "effect")
					result.effect = atoi(param.c_str());
				if(cmd == "effectParam")
					result.effectParam = atoi(param.c_str());
			}
		}
		return result;
	}
	void loadFromList(const vector<string>& list)
	{
		_notes.clear();

		unsigned long i;
		string cmd, param;

		for(i=0;i<list.size();i++)
		{
			if(parse_line(list[i],cmd,param))	// otherwise: empty or comment
			{
				if(cmd == "note")
				{
					vector<string> *nlist = getSubStringSeq(&list,i);
					Note note = loadNote(*nlist);
					_notes.push_back(note);
					delete nlist;
				}
			}
		}
	}
	vector<string> *saveToList()
	{
		vector<string> *result = new vector<string>;

		for(int i=0;i < _notes.size(); i++)
		{
			sqprintf(result,"note=%ld",i);

			Note& note = _notes[i];

			vector<string> noteSave;
			if(note.note)
				sqprintf(&noteSave,"note=%d",note.note);
			if(note.instrument)
				sqprintf(&noteSave,"instrument=%d",note.instrument);
			if(note.velocity)
				sqprintf(&noteSave,"velocity=%d",note.velocity);
			if(note.effect)
				sqprintf(&noteSave,"effect=%d",note.effect);
			if(note.effectParam)
				sqprintf(&noteSave,"effectParam=%d",note.effectParam);

			addSubStringSeq(result,&noteSave);
		}
		return result;
	}
};

REGISTER_IMPLEMENTATION(Track_impl);

class Pattern_impl : virtual public Pattern_skel {
	vector<Track> _tracks;
public:
	Pattern_impl()
	{
		trackCount(8);
	}
	long trackCount() { return _tracks.size(); }
	void trackCount(long size) {
		if(size < _tracks.size())
			_tracks.resize(size);
		else
		{
			while(size > _tracks.size())
			{
				Track t;
				_tracks.push_back(t);
			}
		}
	}
	Track getTrack(long pos)
	{
		assert(pos >= 0);
		assert(pos < (long)_tracks.size());

		return _tracks[pos];
	}
	void loadFromList(const vector<string>& list)
	{
		_tracks.clear();

		unsigned long i;
		string cmd, param;

		for(i=0;i<list.size();i++)
		{
			if(parse_line(list[i],cmd,param))	// otherwise: empty or comment
			{
				if(cmd == "track")
				{
					Track track;
					vector<string> *plist = getSubStringSeq(&list,i);
					track.loadFromList(*plist);
					_tracks.push_back(track);
					delete plist;
				}
			}
		}
	}
	vector<string> *saveToList()
	{
		vector<string> *list = new vector<string>;
		unsigned long i;

		for(i=0; i < trackCount(); i++)
		{
			sqprintf(list,"track=%ld",i);

			vector<string> *portlist = _tracks[i].saveToList();
			addSubStringSeq(list,portlist);
			delete portlist;
		}
		return list;
	}
};
REGISTER_IMPLEMENTATION(Pattern_impl);

class Song_impl : virtual public Song_skel {
	vector<long> songPattern;
	vector<Pattern> pattern;
public:
	Song_impl()
	{
		songLength(1);
		patternCount(1);
	}

	void clear()
	{
		songPattern.clear();
		pattern.clear();
		songLength(1);
		patternCount(1);
	}

	long getSongPattern(long pos)
	{
		assert(pos >= 0);
		if(pos >= songPattern.size())
			songLength(pos+1);

		return songPattern[pos];
	}
	void setSongPattern(long pos, long number)
	{
		assert(pos >= 0);
		if(pos >= songPattern.size())
			songLength(pos+1);

		songPattern[pos] = number;
	}

	long songLength() { return songPattern.size(); }
	void songLength(long size) {
		if(size < songPattern.size())
			songPattern.resize(size);
		else
		{
			while(size > songPattern.size())
				songPattern.push_back(0);
		}
	}

	long patternCount() { return pattern.size(); }

	void patternCount(long size)
	{
		if(size < pattern.size())
			pattern.resize(size);
		else
		{
			while(size > pattern.size())
			{
				Pattern p;
				assert(!p.isNull());
				pattern.push_back(p);
			}
		}
	}

	Pattern getPattern(long number)
	{
		assert(number >= 0);

		if(number >= pattern.size())
			patternCount(number+1);
		return pattern[number];
	}

	vector<string> *saveToList()
	{
		vector<string> *list = new vector<string>;
		unsigned long i;

		for(i=0; i < patternCount(); i++)
		{
			sqprintf(list,"pattern=%ld",i);

			vector<string> *portlist = pattern[i].saveToList();
			addSubStringSeq(list,portlist);
			delete portlist;
		}
		for(i=0; i < songLength(); i++)
		{
			sqprintf(list,"songPattern=%ld",i);
			vector<string> spSave;
			sqprintf(&spSave,"pattern=%ld",songPattern[i]);
			addSubStringSeq(list,&spSave);
		}
		return list;
	}

	long loadSongPattern(const vector<string>& list)
	{
		long result = 0;

		unsigned long i;
		string cmd, param;

		for(i=0;i<list.size();i++)
		{
			if(parse_line(list[i],cmd,param))	// otherwise: empty or comment
			{
				if(cmd == "pattern")
					result = atoi(param.c_str());
			}
		}
		return result;
	}

	void loadFromList(const vector<string>& list)
	{
		pattern.clear();
		songPattern.clear();

		unsigned long i;
		string cmd, param;

		for(i=0;i<list.size();i++)
		{
			if(parse_line(list[i],cmd,param))	// otherwise: empty or comment
			{
				if(cmd == "pattern")
				{
					Pattern p;

					vector<string> *plist = getSubStringSeq(&list,i);
					p.loadFromList(*plist);
					delete plist;

					pattern.push_back(p);
				}
				else if(cmd == "songPattern")
				{
					vector<string> *splist = getSubStringSeq(&list,i);
					songPattern.push_back(loadSongPattern(*splist));
					delete splist;
				}
			}
		}
	}
};
REGISTER_IMPLEMENTATION(Song_impl);

};
};
